@@ -28,25 +28,20 @@
 #  define MRN_HAVE_MYSQL_TYPE_TIME2
 #endif
 
+#if MYSQL_VERSION_ID >= 50709 && !defined(MRN_MARIADB_P)
+#  define MRN_HAVE_MYSQL_TYPE_JSON
+#endif
+
 #if MYSQL_VERSION_ID < 50603
   typedef MYSQL_ERROR Sql_condition;
 #endif
 
 #if defined(MRN_MARIADB_P)
-#  if MYSQL_VERSION_ID >= 50302 && MYSQL_VERSION_ID < 100000
+#  if MYSQL_VERSION_ID < 100000
      typedef COST_VECT Cost_estimate;
 #  endif
 #endif
 
-#if MYSQL_VERSION_ID >= 50516
-#  define MRN_PLUGIN_HAVE_FLAGS 1
-#endif
-
-// for MySQL < 5.5
-#ifndef MY_ALL_CHARSETS_SIZE
-#  define MY_ALL_CHARSETS_SIZE 256
-#endif
-
 #ifndef MRN_MARIADB_P
   typedef char *range_id_t;
 #endif
@@ -76,17 +71,14 @@
 #  define GTS_TABLE 0
 #endif
 
-/* For MySQL 5.1. MySQL 5.1 doesn't have FN_LIBCHAR2. */
-#ifndef FN_LIBCHAR2
-#  define FN_LIBCHAR2 FN_LIBCHAR
-#endif
-
 #if MYSQL_VERSION_ID >= 50607
 #  if MYSQL_VERSION_ID >= 100007 && defined(MRN_MARIADB_P)
 #    define MRN_GET_ERROR_MESSAGE thd_get_error_message(current_thd)
+#    define MRN_GET_ERROR_NUMBER thd_get_error_number(current_thd)
 #    define MRN_GET_CURRENT_ROW_FOR_WARNING(thd) thd_get_error_row(thd)
 #  else
 #    define MRN_GET_ERROR_MESSAGE current_thd->get_stmt_da()->message()
+#    define MRN_GET_ERROR_NUMBER current_thd->get_stmt_da()->sql_errno()
 #    if MYSQL_VERSION_ID >= 50706
 #      define MRN_GET_CURRENT_ROW_FOR_WARNING(thd) \
   thd->get_stmt_da()->current_row_for_condition()
@@ -96,20 +88,16 @@
 #    endif
 #  endif
 #else
-#  if MYSQL_VERSION_ID >= 50500
-#    define MRN_GET_ERROR_MESSAGE current_thd->stmt_da->message()
-#    define MRN_GET_CURRENT_ROW_FOR_WARNING(thd) thd->warning_info->current_row_for_warning()
-#  else
-#    define MRN_GET_ERROR_MESSAGE current_thd->main_da.message()
-#    define MRN_GET_CURRENT_ROW_FOR_WARNING(thd) thd->row_count
-#  endif
+#  define MRN_GET_ERROR_MESSAGE current_thd->stmt_da->message()
+#  define MRN_GET_ERROR_NUMBER current_thd->stmt_da->sql_errno()
+#  define MRN_GET_CURRENT_ROW_FOR_WARNING(thd) thd->warning_info->current_row_for_warning()
 #endif
 
 #if MYSQL_VERSION_ID >= 50607 && !defined(MRN_MARIADB_P)
 #  define MRN_ITEM_HAVE_ITEM_NAME
 #endif
 
-#if MYSQL_VERSION_ID >= 50500 && MYSQL_VERSION_ID < 100000
+#if MYSQL_VERSION_ID < 100000
 #  define MRN_HAVE_TABLE_DEF_CACHE
 #endif
 
@@ -129,10 +117,6 @@
 #  define MRN_TABLE_SHARE_HAVE_LOCK_SHARE
 #endif
 
-#if MYSQL_VERSION_ID >= 50404
-#  define MRN_TABLE_SHARE_HAVE_LOCK_HA_DATA
-#endif
-
 #ifndef TIME_FUZZY_DATE
 /* For MariaDB 10. */
 #  ifdef TIME_FUZZY_DATES
@@ -222,4 +206,87 @@
 #  define MRN_FORMAT_STRING_LENGTH "u"
 #endif
 
+#ifdef MRN_MARIADB_P
+#  define MRN_SUPPORT_CUSTOM_OPTIONS
+#endif
+
+#if defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100000
+#  if MYSQL_VERSION_ID >= 100104
+#    define mrn_init_sql_alloc(thd, mem_root)                           \
+  init_sql_alloc(mem_root,                                              \
+                 TABLE_ALLOC_BLOCK_SIZE,                                \
+                 0,                                                     \
+                 MYF(thd->slave_thread ? 0 : MY_THREAD_SPECIFIC))
+#  else
+#    define mrn_init_sql_alloc(thd, mem_root)           \
+  init_sql_alloc(mem_root,                              \
+                 TABLE_ALLOC_BLOCK_SIZE,                \
+                 0,                                     \
+                 MYF(0))
+#  endif
+#else
+#  if MYSQL_VERSION_ID >= 50709
+#    define mrn_init_sql_alloc(thd, mem_root)           \
+  init_sql_alloc(mrn_memory_key,                        \
+                 mem_root,                              \
+                 TABLE_ALLOC_BLOCK_SIZE,                \
+                 0)
+#  else
+#    define mrn_init_sql_alloc(thd, mem_root)           \
+  init_sql_alloc(mem_root,                              \
+                 TABLE_ALLOC_BLOCK_SIZE,                \
+                 0)
+#  endif
+#endif
+
+#ifdef MRN_MARIADB_P
+#  define MRN_ABORT_ON_WARNING(thd) thd->abort_on_warning
+#else
+#  if MYSQL_VERSION_ID >= 50706
+#    define MRN_ABORT_ON_WARNING(thd) false
+#  else
+#    define MRN_ABORT_ON_WARNING(thd) thd->abort_on_warning
+#  endif
+#endif
+
+#define MRN_ERROR_CODE_DATA_TRUNCATE(thd)                               \
+  (MRN_ABORT_ON_WARNING(thd) ? ER_WARN_DATA_OUT_OF_RANGE : WARN_DATA_TRUNCATED)
+
+#if MYSQL_VERSION_ID >= 50709 && !defined(MRN_MARIADB_P)
+#  define mrn_my_hash_init(hash,                        \
+                           charset,                     \
+                           default_array_elements,      \
+                           key_offset,                  \
+                           key_length,                  \
+                           get_key,                     \
+                           free_element,                \
+                           flags)                       \
+  my_hash_init(hash,                                    \
+               charset,                                 \
+               default_array_elements,                  \
+               key_offset,                              \
+               key_length,                              \
+               get_key,                                 \
+               free_element,                            \
+               flags,                                   \
+               mrn_memory_key)
+#else
+#  define mrn_my_hash_init(hash,                        \
+                           charset,                     \
+                           default_array_elements,      \
+                           key_offset,                  \
+                           key_length,                  \
+                           get_key,                     \
+                           free_element,                \
+                           flags)                       \
+  my_hash_init(hash,                                    \
+               charset,                                 \
+               default_array_elements,                  \
+               key_offset,                              \
+               key_length,                              \
+               get_key,                                 \
+               free_element,                            \
+               flags)
+#endif
+
 #endif /* MRN_MYSQL_COMPAT_H_ */
